//=========================================================================
//
//	FILE:				CP_MouseEvents_View.cpp
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to display mouse events information. 
//	
//	HISTORY:			1st Version - August 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2005-08-30 22:33:04 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// HelloWorld
#include "CP_MouseEvents_View.h"

	// CPLAT_Framework
#include "CP_Enums.h"
#include "CP_Graphics.h"
#include "CP_Utility.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_MouseEvents_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
CP_MouseEvents_View::CP_MouseEvents_View( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true,
			GetFlags() | CPLAT::CP_E_ViewFlags_SupportsMouseTracking ),
				fMouseEventStatus(kMouseEventsView_NoEvent), fStrokeColor( 0.0, 0.0, 0.0, 1.0 )
{
}

//=========================================================================
//	FUNCTION:	~CP_MouseEvents_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_MouseEvents_View::~CP_MouseEvents_View() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we display the coordinates of the mouse when it moves over the
*	bounds of this view.
*
*	@param		inCanvas			The CP_Canvas to draw into.
*	@param		inUpdateArea		The area that needs to be drawn.
*	
*	@exception	none
*
*	@return		void
*/
void CP_MouseEvents_View::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect			localBounds = GetLocalBounds();
	CPLAT::CP_String		text( CP_T("") );
	CPLAT::CP_Graphics		graphics( inCanvas );
	CPLAT::CP_FontSpec		fontSpec;
	
	graphics.SetStrokeColor( fStrokeColor );
	graphics.StrokeRect( localBounds );
	
	localBounds.Inset( 1, 1 );
	
	switch( fMouseEventStatus )
	{
		case kMouseEventsView_MouseMoved:
			{
				text = CP_T("Coords: (H/V): ");
				text += CPLAT::CP_Utility::CP_FloatToString( fMouseLocation.h() );
				text += CP_T(" / ");
				text += CPLAT::CP_Utility::CP_FloatToString( fMouseLocation.v() );
			}
			break;
	}
		
	graphics.DrawText( fontSpec, text, localBounds, CPLAT::CP_E_TextAlign_HorCenter, CPLAT::CP_E_TextAlign_VerMiddle );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mouse Events ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleMouseMoved
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Event called when the mouse moves over this view.
*
*	@param		inEvent					The mouse moved event.
*	@param		inLocalMouseLocation	The location of the mouse in local coordinates.
*	
*	@exception	none
*
*	@return		void
*/
void CP_MouseEvents_View::HandleMouseMoved( CPLAT::CP_Event_Base& /*inEvent*/,
	const CPLAT::CP_Point& inLocalMouseLocation )
{
	fMouseLocation = inLocalMouseLocation;
	
	fMouseEventStatus = kMouseEventsView_MouseMoved;

	Refresh();
}

//=========================================================================
//	FUNCTION:	HandleMouseEntered
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Event called when the mouse enters this view. We set the color displayed
*	by the frame of this view to red to indicate the mouse has entered the
*	views bounds.
*
*	@param		inEvent					The mouse entered event.
*	
*	@exception	none
*
*	@return		void
*/
void CP_MouseEvents_View::HandleMouseEntered( CPLAT::CP_Event_Base& /*inEvent*/ )
{	
	fMouseEventStatus = kMouseEventsView_MouseEntered;
	
	fStrokeColor.Set( 1.0, 0.0, 0.0, 1.0 );

	Refresh();
}

//=========================================================================
//	FUNCTION:	HandleMouseEntered
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Event called when the mouse exits this view. We set the color displayed
*	by the frame of this view to blue to indicate the mouse has exited the
*	views bounds.
*
*	@param		inEvent					The mouse exited event.
*	
*	@exception	none
*
*	@return		void
*/
void CP_MouseEvents_View::HandleMouseExited( CPLAT::CP_Event_Base& /*inEvent*/ )
{
	fMouseEventStatus = kMouseEventsView_MouseExited;
	
	fStrokeColor.Set( 0.0, 0.0, 1.0, 1.0 );

	Refresh();
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
