//=========================================================================
//
//	FILE:				CP_Color.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to manage named colors.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Color
#define _H_CP_Color
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_ARGBColor.h"
#include "CP_RGBColor.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Color
*	@brief		General class to represent a color.
*
*				This class is used to maintain a list of colors that can be accessed
*				by name, or index into the table.
*
*				Support for setting and getting the HLS values of the color, and setting
*				and getting the color as a CP_RGBColor are also implemented.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_Color : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Color();
										CP_Color(
												UInt16 inRed,
												UInt16 inGreen,
												UInt16 inBlue );
										CP_Color( const CP_Color& inColor );
										CP_Color( UInt32 inColorRef );
	//@}
	//! Destructor.
	virtual								~CP_Color();
	//@}

	//! @name Getters
	//@{
			UInt16						Red() const;
			UInt16						Green() const;
			UInt16						Blue() const;
			
			float						Hue() const;
			float						Luminance() const;
			float						Saturation() const;

			CP_RGBColor					GetRGBColor();
			CP_ARGBColor				GetARGBColor();
	//@}
			
	//! @name Named colors
	//@{
			CP_String					GetString() const;
			bool    					SetString( const CP_String& inColor );
			
			CP_String					GetName() const;

	static	CP_Color					GetColor( const CP_String& inColor ); 
	static	CP_Color					GetColor( SInt16 inindex );
	static	CP_String					GetName( SInt16 inindex );

										/*!
										*	Returns the number of colors we know the names of.
										*	
										*	@exception	none
										*	
										*	@return		SInt16		The number of colors in the table.
										*/
	static	SInt16    					GetCount()									{ return( numNamedColors ); }
	//@}

	//! @name operators
	//@{
			bool						operator == ( const CP_Color& inColor ) const;
			bool						operator != ( const CP_Color& inColor ) const;
	
			CP_Color&					operator = ( const CP_Color& inColor );
	//@}

	//! @name Setters
	//@{
			void						Set(
												UInt16 inRed,
												UInt16 inGreen,
												UInt16 inBlue );
		
										/*!
										*	Sets the color from the passed in CP_Color object.
										*	
										*	@param		inColor			CP_Color object to copy information from.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Set( const CP_Color& inColor )				{ operator = ( inColor ); }

			void						SetHue( float inHue );
			void						SetLuminance( float inLuminance );
			void						SetSaturation( float inSaturation );
			void						SetHLS(
												float inHue,
												float inLuminance,
												float inSaturation );
	//@}

private:
	//! @name Getters
	//@{
  			void						ToHLS();
			void						ToRGB();
			unsigned char				ToRGB(
												float rm1,
												float rm2,
												float rh );
	//@}

public:
		// standard named colors html / gdk
	enum CP_E_NAMED_CP_COLOR
	{
		cp_none                 = 0x7FFFFFFF,
	    cp_aliceblue            = 0x00FFF8F0,   // RGB(0xF0, 0xF8, 0xFF) 
	    cp_antiquewhite         = 0x00D7EBFA,   // RGB(0xFA, 0xEB, 0xD7) 
	    cp_aqua                 = 0x00FFFF00,   // RGB(0x00, 0xFF, 0xFF) 
	    cp_aquamarine           = 0x00D4FF7F,   // RGB(0x7F, 0xFF, 0xD4) 
	    cp_azure                = 0x00FFFFF0,   // RGB(0xF0, 0xFF, 0xFF) 
	    cp_beige                = 0x00DCF5F5,   // RGB(0xF5, 0xF5, 0xDC) 
	    cp_bisque               = 0x00C4E4FF,   // RGB(0xFF, 0xE4, 0xC4) 
	    cp_black                = 0x00000000,   // RGB(0x00, 0x00, 0x00) 
	    cp_blanchedalmond       = 0x00CDEBFF,   // RGB(0xFF, 0xEB, 0xCD) 
	    cp_blue                 = 0x00FF0000,   // RGB(0x00, 0x00, 0xFF) 
	    cp_blueviolet           = 0x00E22B8A,   // RGB(0x8A, 0x2B, 0xE2) 
	    cp_brown                = 0x002A2AA5,   // RGB(0xA5, 0x2A, 0x2A) 
	    cp_burlywood            = 0x0087B8DE,   // RGB(0xDE, 0xB8, 0x87) 
	    cp_cadetblue            = 0x00A09E5F,   // RGB(0x5F, 0x9E, 0xA0) 
	    cp_chartreuse           = 0x0000FF7F,   // RGB(0x7F, 0xFF, 0x00) 
	    cp_chocolate            = 0x001E69D2,   // RGB(0xD2, 0x69, 0x1E) 
	    cp_coral                = 0x00507FFF,   // RGB(0xFF, 0x7F, 0x50) 
	    cp_cornflower           = 0x00ED9564,   // RGB(0x64, 0x95, 0xED) 
	    cp_cornsilk             = 0x00DCF8FF,   // RGB(0xFF, 0xF8, 0xDC) 
	    cp_crimson              = 0x003C14DC,   // RGB(0xDC, 0x14, 0x3C) 
	    cp_cyan                 = 0x00FFFF00,   // RGB(0x00, 0xFF, 0xFF) 
	    cp_darkblue             = 0x008B0000,   // RGB(0x00, 0x00, 0x8B) 
	    cp_darkcyan             = 0x008B8B00,   // RGB(0x00, 0x8B, 0x8B) 
	    cp_darkgoldenrod        = 0x000B86B8,   // RGB(0xB8, 0x86, 0x0B) 
	    cp_darkgray             = 0x00A9A9A9,   // RGB(0xA9, 0xA9, 0xA9) 
	    cp_darkgreen            = 0x00006400,   // RGB(0x00, 0x64, 0x00) 
	    cp_darkkhaki            = 0x006BB7BD,   // RGB(0xBD, 0xB7, 0x6B) 
	    cp_darkmagenta          = 0x008B008B,   // RGB(0x8B, 0x00, 0x8B) 
	    cp_darkolivegreen       = 0x002F6B55,   // RGB(0x55, 0x6B, 0x2F) 
	    cp_darkorange           = 0x00008CFF,   // RGB(0xFF, 0x8C, 0x00) 
	    cp_darkorchid           = 0x00CC3299,   // RGB(0x99, 0x32, 0xCC) 
	    cp_darkred              = 0x0000008B,   // RGB(0x8B, 0x00, 0x00) 
	    cp_darksalmon           = 0x007A96E9,   // RGB(0xE9, 0x96, 0x7A) 
	    cp_darkseagreen         = 0x008BBC8F,   // RGB(0x8F, 0xBC, 0x8B) 
	    cp_darkslateblue        = 0x008B3D48,   // RGB(0x48, 0x3D, 0x8B) 
	    cp_darkslategray        = 0x004F4F2F,   // RGB(0x2F, 0x4F, 0x4F) 
	    cp_darkturquoise        = 0x00D1CE00,   // RGB(0x00, 0xCE, 0xD1) 
	    cp_darkviolet           = 0x00D30094,   // RGB(0x94, 0x00, 0xD3) 
	    cp_deeppink             = 0x009314FF,   // RGB(0xFF, 0x14, 0x93) 
	    cp_deepskyblue          = 0x00FFBF00,   // RGB(0x00, 0xBF, 0xFF) 
	    cp_dimgray              = 0x00696969,   // RGB(0x69, 0x69, 0x69) 
	    cp_dodgerblue           = 0x00FF901E,   // RGB(0x1E, 0x90, 0xFF) 
	    cp_firebrick            = 0x002222B2,   // RGB(0xB2, 0x22, 0x22) 
	    cp_floralwhite          = 0x00F0FAFF,   // RGB(0xFF, 0xFA, 0xF0) 
	    cp_forestgreen          = 0x00228B22,   // RGB(0x22, 0x8B, 0x22) 
	    cp_fuchsia              = 0x00FF00FF,   // RGB(0xFF, 0x00, 0xFF) 
	    cp_gainsboro            = 0x00DCDCDC,   // RGB(0xDC, 0xDC, 0xDC) 
	    cp_ghostwhite           = 0x00FFF8F8,   // RGB(0xF8, 0xF8, 0xFF) 
	    cp_gold                 = 0x0000D7FF,   // RGB(0xFF, 0xD7, 0x00) 
	    cp_goldenrod            = 0x0020A5DA,   // RGB(0xDA, 0xA5, 0x20) 
	    cp_gray                 = 0x00808080,   // RGB(0x80, 0x80, 0x80) 
	    cp_green                = 0x00008000,   // RGB(0x00, 0x80, 0x00) 
	    cp_greenyellow          = 0x002FFFAD,   // RGB(0xAD, 0xFF, 0x2F) 
	    cp_honeydew             = 0x00F0FFF0,   // RGB(0xF0, 0xFF, 0xF0) 
	    cp_hotpink              = 0x00B469FF,   // RGB(0xFF, 0x69, 0xB4) 
	    cp_indianred            = 0x005C5CCD,   // RGB(0xCD, 0x5C, 0x5C) 
	    cp_indigo               = 0x0082004B,   // RGB(0x4B, 0x00, 0x82) 
	    cp_ivory                = 0x00F0FFFF,   // RGB(0xFF, 0xFF, 0xF0) 
	    cp_khaki                = 0x008CE6F0,   // RGB(0xF0, 0xE6, 0x8C) 
	    cp_lavender             = 0x00FAE6E6,   // RGB(0xE6, 0xE6, 0xFA) 
	    cp_lavenderblush        = 0x00F5F0FF,   // RGB(0xFF, 0xF0, 0xF5) 
	    cp_lawngreen            = 0x0000FC7C,   // RGB(0x7C, 0xFC, 0x00) 
	    cp_lemonchiffon         = 0x00CDFAFF,   // RGB(0xFF, 0xFA, 0xCD) 
	    cp_lightblue            = 0x00E6D8AD,   // RGB(0xAD, 0xD8, 0xE6) 
	    cp_lightcoral           = 0x008080F0,   // RGB(0xF0, 0x80, 0x80) 
	    cp_lightcyan            = 0x00FFFFE0,   // RGB(0xE0, 0xFF, 0xFF) 
	    cp_lightgoldenrodyellow = 0x00D2FAFA,   // RGB(0xFA, 0xFA, 0xD2) 
	    cp_lightgreen           = 0x0090EE90,   // RGB(0x90, 0xEE, 0x90) 
	    cp_lightgrey            = 0x00D3D3D3,   // RGB(0xD3, 0xD3, 0xD3) 
	    cp_lightpink            = 0x00C1B6FF,   // RGB(0xFF, 0xB6, 0xC1) 
	    cp_lightsalmon          = 0x007AA0FF,   // RGB(0xFF, 0xA0, 0x7A) 
	    cp_lightseagreen        = 0x00AAB220,   // RGB(0x20, 0xB2, 0xAA) 
	    cp_lightskyblue         = 0x00FACE87,   // RGB(0x87, 0xCE, 0xFA) 
	    cp_lightslategray       = 0x00998877,   // RGB(0x77, 0x88, 0x99) 
	    cp_lightsteelblue       = 0x00DEC4B0,   // RGB(0xB0, 0xC4, 0xDE) 
	    cp_lightyellow          = 0x00E0FFFF,   // RGB(0xFF, 0xFF, 0xE0) 
	    cp_lime                 = 0x0000FF00,   // RGB(0x00, 0xFF, 0x00) 
	    cp_limegreen            = 0x0032CD32,   // RGB(0x32, 0xCD, 0x32) 
	    cp_linen                = 0x00E6F0FA,   // RGB(0xFA, 0xF0, 0xE6) 
	    cp_magenta              = 0x00FF00FF,   // RGB(0xFF, 0x00, 0xFF) 
	    cp_maroon               = 0x00000080,   // RGB(0x80, 0x00, 0x00) 
	    cp_mediumaquamarine     = 0x00AACD66,   // RGB(0x66, 0xCD, 0xAA) 
	    cp_mediumblue           = 0x00CD0000,   // RGB(0x00, 0x00, 0xCD) 
	    cp_mediumorchid         = 0x00D355BA,   // RGB(0xBA, 0x55, 0xD3) 
	    cp_mediumpurple         = 0x00DB7093,   // RGB(0x93, 0x70, 0xDB) 
	    cp_mediumseagreen       = 0x0071B33C,   // RGB(0x3C, 0xB3, 0x71) 
	    cp_mediumslateblue      = 0x00EE687B,   // RGB(0x7B, 0x68, 0xEE) 
	    cp_mediumspringgreen    = 0x009AFA00,   // RGB(0x00, 0xFA, 0x9A) 
	    cp_mediumturquoise      = 0x00CCD148,   // RGB(0x48, 0xD1, 0xCC) 
	    cp_mediumvioletred      = 0x008515C7,   // RGB(0xC7, 0x15, 0x85) 
	    cp_midnightblue         = 0x00701919,   // RGB(0x19, 0x19, 0x70) 
	    cp_mintcream            = 0x00FAFFF5,   // RGB(0xF5, 0xFF, 0xFA) 
	    cp_mistyrose            = 0x00E1E4FF,   // RGB(0xFF, 0xE4, 0xE1) 
	    cp_moccasin             = 0x00B5E4FF,   // RGB(0xFF, 0xE4, 0xB5) 
	    cp_navajowhite          = 0x00ADDEFF,   // RGB(0xFF, 0xDE, 0xAD) 
	    cp_navy                 = 0x00800000,   // RGB(0x00, 0x00, 0x80) 
	    cp_oldlace              = 0x00E6F5FD,   // RGB(0xFD, 0xF5, 0xE6) 
	    cp_olive                = 0x00008080,   // RGB(0x80, 0x80, 0x00) 
	    cp_olivedrab            = 0x00238E6B,   // RGB(0x6B, 0x8E, 0x23) 
	    cp_orange               = 0x0000A5FF,   // RGB(0xFF, 0xA5, 0x00) 
	    cp_orangered            = 0x000045FF,   // RGB(0xFF, 0x45, 0x00) 
	    cp_orchid               = 0x00D670DA,   // RGB(0xDA, 0x70, 0xD6) 
	    cp_palegoldenrod        = 0x00AAE8EE,   // RGB(0xEE, 0xE8, 0xAA) 
	    cp_palegreen            = 0x0098FB98,   // RGB(0x98, 0xFB, 0x98) 
	    cp_paleturquoise        = 0x00EEEEAF,   // RGB(0xAF, 0xEE, 0xEE) 
	    cp_palevioletred        = 0x009370DB,   // RGB(0xDB, 0x70, 0x93) 
	    cp_papayawhip           = 0x00D5EFFF,   // RGB(0xFF, 0xEF, 0xD5) 
	    cp_peachpuff            = 0x00B9DAFF,   // RGB(0xFF, 0xDA, 0xB9) 
	    cp_peru                 = 0x003F85CD,   // RGB(0xCD, 0x85, 0x3F) 
	    cp_pink                 = 0x00CBC0FF,   // RGB(0xFF, 0xC0, 0xCB) 
	    cp_plum                 = 0x00DDA0DD,   // RGB(0xDD, 0xA0, 0xDD) 
	    cp_powderblue           = 0x00E6E0B0,   // RGB(0xB0, 0xE0, 0xE6) 
	    cp_purple               = 0x00800080,   // RGB(0x80, 0x00, 0x80) 
	    cp_red                  = 0x000000FF,   // RGB(0xFF, 0x00, 0x00) 
	    cp_rosybrown            = 0x008F8FBC,   // RGB(0xBC, 0x8F, 0x8F) 
	    cp_royalblue            = 0x00E16941,   // RGB(0x41, 0x69, 0xE1) 
	    cp_saddlebrown          = 0x0013458B,   // RGB(0x8B, 0x45, 0x13) 
	    cp_salmon               = 0x007280FA,   // RGB(0xFA, 0x80, 0x72) 
	    cp_sandybrown           = 0x0060A4F4,   // RGB(0xF4, 0xA4, 0x60) 
	    cp_seagreen             = 0x00578B2E,   // RGB(0x2E, 0x8B, 0x57) 
	    cp_seashell             = 0x00EEF5FF,   // RGB(0xFF, 0xF5, 0xEE) 
	    cp_sienna               = 0x002D52A0,   // RGB(0xA0, 0x52, 0x2D) 
	    cp_silver               = 0x00C0C0C0,   // RGB(0xC0, 0xC0, 0xC0) 
	    cp_skyblue              = 0x00EBCE87,   // RGB(0x87, 0xCE, 0xEB) 
	    cp_slateblue            = 0x00CD5A6A,   // RGB(0x6A, 0x5A, 0xCD) 
	    cp_slategray            = 0x00908070,   // RGB(0x70, 0x80, 0x90) 
	    cp_snow                 = 0x00FAFAFF,   // RGB(0xFF, 0xFA, 0xFA) 
	    cp_springgreen          = 0x007FFF00,   // RGB(0x00, 0xFF, 0x7F) 
	    cp_steelblue            = 0x00B48246,   // RGB(0x46, 0x82, 0xB4) 
	    cp_tan                  = 0x008CB4D2,   // RGB(0xD2, 0xB4, 0x8C) 
	    cp_teal                 = 0x00808000,   // RGB(0x00, 0x80, 0x80) 
	    cp_thistle              = 0x00D8BFD8,   // RGB(0xD8, 0xBF, 0xD8) 
	    cp_tomato               = 0x004763FF,   // RGB(0xFF, 0x63, 0x47) 
	    cp_turquoise            = 0x00D0E040,   // RGB(0x40, 0xE0, 0xD0) 
	    cp_violet               = 0x00EE82EE,   // RGB(0xEE, 0x82, 0xEE) 
	    cp_wheat                = 0x00B3DEF5,   // RGB(0xF5, 0xDE, 0xB3) 
	    cp_white                = 0x00FFFFFF,   // RGB(0xFF, 0xFF, 0xFF) 
	    cp_whitesmoke           = 0x00F5F5F5,   // RGB(0xF5, 0xF5, 0xF5) 
	    cp_yellow               = 0x0000FFFF,   // RGB(0xFF, 0xFF, 0x00) 
	    cp_yellowgreen          = 0x0032CD9A,    // RGB(0x9A, 0xCD, 0x32) 

			// Apple Grayscale colors
		cp_gray_1				= 0x00EEEEEE,
		cp_gray_2				= 0x00DDDDDD,
		cp_gray_3				= 0x00CCCCCC,
		cp_gray_4				= 0x00BBBBBB,
		cp_gray_5				= 0x00AAAAAA,
		cp_gray_6				= 0x00999999,
		cp_gray_7				= 0x00888888,
		cp_gray_8				= 0x00777777,
		cp_gray_9				= 0x00666666,
		cp_gray_10				= 0x00555555,
		cp_gray_11				= 0x00444444,
		cp_gray_12				= 0x00222222
	};

	enum CP_E_NAMED_CP_COLOR_INDEX
	{ 
		INDEX_aliceblue = 1, INDEX_antiquewhite, INDEX_aqua, INDEX_aquamarine, INDEX_azure, INDEX_beige, INDEX_bisque, INDEX_black, 
		INDEX_blanchedalmond, INDEX_blue, INDEX_blueviolet, INDEX_brown, INDEX_burlywood, INDEX_cadetblue, INDEX_chartreuse,
		INDEX_chocolate, INDEX_coral, INDEX_cornflower, INDEX_cornsilk, INDEX_crimson, INDEX_cyan, INDEX_darkblue, INDEX_darkcyan, 
		INDEX_darkgoldenrod, INDEX_darkgray, INDEX_darkgreen, INDEX_darkkhaki, INDEX_darkmagenta, INDEX_darkolivegreen, 
		INDEX_darkorange, INDEX_darkorchid, INDEX_darkred, INDEX_darksalmon, INDEX_darkseagreen, INDEX_darkslateblue, 
		INDEX_darkslategray, INDEX_darkturquoise, INDEX_darkviolet, INDEX_deeppink, INDEX_deepskyblue, INDEX_dimgray, 
		INDEX_dodgerblue, INDEX_firebrick, INDEX_floralwhite, INDEX_forestgreen, INDEX_fuchsia, INDEX_gainsboro, 
		INDEX_ghostwhite, INDEX_gold, INDEX_goldenrod, INDEX_gray, INDEX_green, INDEX_greenyellow, INDEX_honeydew, INDEX_hotpink, 
		INDEX_indianred, INDEX_indigo, INDEX_ivory, INDEX_khaki, INDEX_lavender, INDEX_lavenderblush, INDEX_lawngreen, 
		INDEX_lemonchiffon, INDEX_lightblue, INDEX_lightcoral, INDEX_lightcyan, INDEX_lightgoldenrodyellow, 
		INDEX_lightgreen, INDEX_lightgrey, INDEX_lightpink, INDEX_lightsalmon, INDEX_lightseagreen, INDEX_lightskyblue, 
		INDEX_lightslategray, INDEX_lightsteelblue, INDEX_lightyellow, INDEX_lime, INDEX_limegreen, INDEX_linen, 
		INDEX_magenta, INDEX_maroon, INDEX_mediumaquamarine, INDEX_mediumblue, INDEX_mediumorchid, INDEX_mediumpurple, 
		INDEX_mediumseagreen, INDEX_mediumslateblue, INDEX_mediumspringgreen, INDEX_mediumturquoise, 
		INDEX_mediumvioletred, INDEX_midnightblue, INDEX_mintcream, INDEX_mistyrose, INDEX_moccasin, INDEX_navajowhite, 
		INDEX_navy, INDEX_oldlace, INDEX_olive, INDEX_olivedrab, INDEX_orange, INDEX_orangered, INDEX_orchid, INDEX_palegoldenrod, 
		INDEX_palegreen, INDEX_paleturquoise, INDEX_palevioletred, INDEX_papayawhip, INDEX_peachpuff, INDEX_peru, INDEX_pink,
		INDEX_plum, INDEX_powderblue, INDEX_purple, INDEX_red, INDEX_rosybrown, INDEX_royalblue, INDEX_saddlebrown, INDEX_salmon, 
		INDEX_sandybrown, INDEX_seagreen, INDEX_seashell, INDEX_sienna, INDEX_silver, INDEX_skyblue, INDEX_slateblue, 
		INDEX_slategray, INDEX_snow, INDEX_springgreen, INDEX_steelblue, INDEX_tan, INDEX_teal, INDEX_thistle, INDEX_tomato, 
		INDEX_turquoise, INDEX_violet, INDEX_wheat, INDEX_white, INDEX_whitesmoke, INDEX_yellow, INDEX_yellowgreen, INDEX_gray_1,
		INDEX_gray_2, INDEX_gray_3, INDEX_gray_4, INDEX_gray_5, INDEX_gray_6, INDEX_gray_7, INDEX_gray_8, INDEX_gray_9,
		INDEX_gray_10, INDEX_gray_11, INDEX_gray_12, numNamedColors
	};

	// class data
protected:
							
	UInt16								fRed;			/*! The red color component. */
							
	UInt16								fGreen;			/*! The green color component. */
	UInt16								fBlue;			/*! The blue color component. */

	float								fHue;			/*! The hue component */
	float								fSaturation;	/*! The saturation component. */
	float								fLuminance;		/*! The luminance component. */

	bool								fIsRGB; 		/*! Flag to indicate if state is in rgb. */
	bool								fIsHLS; 		/*! Flag to indicate if state is in hls. */

	UInt32								fColorRef;		/*! Internal representation of color. */
	
	/*! Structure to hold the color and a corresponding name. */
	struct CP_S_NamedColor
	{
		UInt32		color; 
		char  		*colorName;
	};
	static const CP_S_NamedColor	fNamedColors[ numNamedColors ]; 

	#if TARGET_OS_LINUX
		GdkColor					*fGDKColor;		/*! The Gdk reprentation of a color. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Color( RGBColor inColor );
	//@}

	//! @name operators
	//@{
										operator RGBColor *();
										operator const RGBColor *() const;
							
										operator RGBColor();
										operator const RGBColor() const;
	//@}

	//! @name Setters
	//@{
			void						Set( const RGBColor inColor );
	//@}	

#endif

#if TARGET_OS_WIN32
public:
	//! @name operators
	//@{
			COLORREF					GetCOLORREF() const;

										/*!
										*	Return the color as a Windows COLORREF.
										*	
										*	@exception	none
										*	
										*	@return		COLORREF		Windows COLORREF of this color.
										*/
										operator COLORREF () const					{ return GetCOLORREF(); }
	//@}

	//! @name Setters
	//@{
			void						Set( COLORREF inColor );
	//@}

#endif

#if TARGET_OS_LINUX
public:
	//! @name Getters
	//@{
										/*!
										*	Return the color as a GdkColor.
										*	
										*	@exception	none
										*	
										*	@return		GdkColor		GdkColor of this color.
										*/
			GdkColor					*GetGDKColor()								{ return( fGDKColor ); }
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Color
