//=========================================================================
//
//	FILE:				CP_HandleBlock.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Allocates relocatable blocks of memory using Handles.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_HandleBlock
#define _H_CP_HandleBlock
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_ReferenceCounted.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_HandleBlock
*	@brief		Memory allocation class based on Handles.
*	
*				An object to manage blocks of memory using handles (i.e. relocatable memory blocks).
*				Note: if handle based memory allocation is not support on the platforms
*				OS, then regular pointer allocation is used.
*/
class CP_EXPORT CP_HandleBlock
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_HandleBlock();
										CP_HandleBlock( UInt32 inSize );
										explicit CP_HandleBlock( const CP_HandleBlock& inHandleBlock );
	//! Destructor.
										~CP_HandleBlock() throw();
	//@}

	//! @name Copying
	//@{
			void						CopyTo(
												void *ioBuffer,
												UInt32 inNumBytes ) const;
			void						CopyFrom(
												const void *inBuffer,
												UInt32 inNumBytes );
	//@}

	//! @name Getters
	//@{
			OSHandle					GetOSHandle() const;
			char						*GetPtr() const;
	//@}

	//! @name Length
	//@{
			UInt32						Length() const;
			void						SetLength( UInt32 inLength );
	//@}
			
	//! @name Locking
	//@{
			void						Lock() const;
			void						Unlock() const;

										/*!
										*	Returns true if the memory block is locked.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is locked.
										*/
			bool						IsLocked() const									{ return( fLocked ); }
	//@}

	//! @name Operators
	//@{
			CP_HandleBlock&				operator = (const CP_HandleBlock& inHandleBlock );
	//@}
			
	//! @name Testers
	//@{
										/*!
										*	Returns true if the memory block is not NULL.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is not NULL.
										*/
			bool						IsValid() const									{ return( fBlock != NULL ); }

										/*!
										*	Returns true if this object owns the block of memory, and is therefore
										*	responsible for its deletion. Otherwise the memory block held by this object
										*	will not be deleted by this class.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if this object owns the memory handle.
										*/
			bool						OwnsHandle() const								{ return( fOwnsHandle ); }
	//@}
			
	//! @name Utility
	//@{
			void						Wrap(
												OSHandle inExistingHandle,
												bool inOwnsHandle = true );


										/*!
										*	Notifies this class that it no longer owns the memory, and will not
										*	delete it. You are now responsible for deleting the memory to avoid a
										*	memory leak.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						Detach()									{ fOwnsHandle = false; }
							
			CP_HandleBlock				*Clone();
	//@}

private:
	//! @name Allocation
	//@{
			void						Allocate( UInt32 inSize );
			void						Deallocate();
	//@}

	// class data
protected:
								
	OSHandle							fBlock;				/*! Platforms memory object.*/
	bool								fOwnsHandle;		/*! Flag to indicate who is responsible for deleting the platforms memory.*/
	mutable bool						fLocked;			/*! Flag to indicate if the block is locked.*/
	UInt32								fActualBlockLength;	/*! Length of the memory block.*/

	#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		mutable char					*fBlockPtr;			/*! Pointer to the memory when the block is locked.*/
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_HandleBlock(
												Handle inMacHandle,
												bool inOwnsHandle = true );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the MacOS Handle.
										*	
										*	@exception	none
										*	
										*	@return		OSHandle
										*/
										operator Handle() const							{ return( fBlock ); }
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_HandleBlock