//=========================================================================
//
//	FILE:				CP_Heap.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to handle management of a MacOS Heap.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 233 $
//						Latest Checkout:  $Date: 2005-07-16 19:41:39 -0700 (Sat, 16 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_Heap
#define _H_CP_Heap
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

 	// CPLAT_Framework
#include "CP_String.h"
#include "CP_Types.h"
    
    // Standard Library
#include <cstdio>
#include <new>

CPLAT_Begin_Namespace_CPLAT

	// function calls to allocate memory in the heap
typedef void	*(*NewFunc)(size_t inSize);
typedef void 	(*DeleteFunc)(void *inPtr);

	// Forward declarations
class CP_Heap;
class CP_File;

extern CP_Heap			*DoCreateCP_Heap();

struct CP_S_MemoryBlock
{
	unsigned			fIsArray 	: 1;
	unsigned			fDataSize   : 31;		// size of our memory allocation
	SInt16				fBlockStatus;			// status of the memory allocation - see CP_MemoryRecording.h for CP_BS
	SInt16				fMemorySession;			// memory session for this allocation
	UInt32				fMarker;
	CP_S_MemoryBlock	*fPrevBlock;
	CP_S_MemoryBlock	*fNextBlock;
	const char			*fFunction;				// function where memory allocated
	const char			*fFile;					// file where memory allocated
	SInt16				fLine;					// line in file where allocated
	unsigned char		fData[4];
};

/*!
*	@class		CP_Heap
*	@brief		Class to handle management of a MacOS Heap.
*/
class CP_EXPORT CP_Heap
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Heap();
	//! Destructor.
										~CP_Heap();
	//@}

	//! @name Instance
	//@{
	static CP_Heap						*Instance();
	//@}

	//! @name Allocate / Dealloc
	//@{
			void						*Allocate(
												UInt32 inSize,
												bool isArray,
												const char *inFunction,
												const char *inFile,
												int inLine );
						
			void 						Deallocate(
												void *inBlock,
												bool isArray );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the current number of bytes allocated.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		Current number of allocated bytes.
										*/
			SInt32 						GetCurrentBytes() const							{ return( fCurrentBytes ); }

										/*!
										*	Returns the highest number of bytes allocated.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		Highest number of allocated bytes.
										*/
			SInt32 						GetHighwaterMark() const						{ return( fHighwaterMark ); }

										/*!
										*	Returns the number of bytes used for debugging purposes.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		Number of bytes used for debugging purposes.
										*/
			SInt32 						GetDebugOverhead() const						{ return( 100 * fDebugOverhead / fCurrentBytes ); }

										/*!
										*	Returns the number of blocks currently marked as leaks.
										*	
										*	@exception	none
										*	
										*	@return		SInt32		Number of blocks currently marked as leaks.
										*/
			SInt32 						GetLeakCount() const							{ return( fLeakCount ); }

										/*!
										*	Double delete detection.
										*	Note: When double delete detection is turn on, the allocated
										*	block is only marked as deleted, but not actually deleted from
										*	memory so memory useage may dramatically increase.
										*
										*	@param		inTurnOn	If true, turn on double delete detection.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
			void						TurnOnDoubleDeleteDetection( bool inTurnOn = true )	{ fDontDeleteBlocks = inTurnOn; }

										/*!
										*	Returns the state of the double delete detection flag.
										*	
										*	@exception	none
										*	
										*	@return		bool		Current state of the double delete detection flag. If
										*							true, then double delete detection is on.
										*/
			bool						GetDoubleDeleteDetectionStatus()					{ return( fDontDeleteBlocks ); }
	//@}
			
	//! @name operators
	//@{
	static	void						*operator new( size_t inSize );
	static	void 						operator delete( void *inPtr );
	//@}

	//! @name Reporting
	//@{
			void						DumpAllAllocations( CP_File& inFile );
			void						DumpSessionAllocations(
												CP_File& outFile,
												SInt16 inSessionIndex );
			UInt16						CountSessionLeaks( SInt16 inSessionIndex );

			void						MarkAllBlocksNonLeaks();
	//@}

	//! @name Validators
	//@{
			void 						ValidateBlock( const void *inPtr ) const;
		 	void 						ValidateHeap() const;
	//@}
	
protected:			
	//! @name Recording
	//@{
			void 						AddDebugInfo(
												CP_S_MemoryBlock *inBlock,
												UInt32 inAllocSize,
												bool inIsArray,
												const char *inFunction,
												const char *inFile,
												int inLine );
	//@}
			
private:
	//! @name Recording
	//@{
			void						FillBlock(
												void *returnPtr,
												UInt16 inBlockSize );

			void						DumpOneLeak(
												const void *inPtr,
												CP_File& inFile );
	//@}

	//! @name Constructor / Destructor
	//@{
										CP_Heap( const CP_Heap& inHeap );
	//@}

	//! @name operators
	//@{
			CP_Heap&					operator = ( const CP_Heap& inHeap );
	//@}

	// Data members
protected:
								
	static CP_Heap						*gInstance;			/*! Points to the single instance of ourselves.*/

	CP_S_MemoryBlock					*fFirstBlock;

	SInt32								fBlockCount;
	SInt32								fLeakCount;

	SInt32								fCurrentBytes;
	SInt32								fHighwaterMark;			
	SInt32 								fDebugOverhead;

	bool								fDontDeleteBlocks;	/*! Flag to indicate if we delete blocks, or just mark them for double delete detection.*/

public:
	static NewFunc						gNewFunc;			/*! Pointer to the function used to allocate memory.*/
	static DeleteFunc					gDeleteFunc;		/*! Pointer to the function to deallocate memory.*/


//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

// ===================================================================================
//	class CP_HeapLock
// ===================================================================================
/*!
*	@class		CP_HeapLock
*	@brief		Class to handle locking heap for threading support.
*/
class CP_EXPORT CP_HeapLock
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_HeapLock();
	//! Destructor.
										~CP_HeapLock();
	//@}
					
private:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_HeapLock( const CP_HeapLock& inHeap );
	//@}

	//! @name operators
	//@{
			CP_HeapLock& 				operator = ( const CP_HeapLock& inHeap );
	//@}
				
	// Data members
protected:
	#if TARGET_OS_WIN32
		static CRITICAL_SECTION			gObject;
		static bool						gInited;
	#endif
	#if TARGET_OS_MAC
		static ThreadID					gThread;
		static SInt32					gLockCount;
	#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Heap
