//=========================================================================
//
//	FILE:				CP_M_CommandHandler.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Classes used in CPLAT to manage the status and handling
//						of commands.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 162 $
//						Latest Checkout:  $Date: 2005-04-28 15:05:07 -0700 (Thu, 28 Apr 2005) $
//
//=========================================================================
#ifndef _H_CP_M_CommandHandler
#define _H_CP_M_CommandHandler
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_CommandNos.h"
#if !CP_CORE_ONLY // JUM 2004-11-25
	#include "CP_Menu.h"
#endif
#include "CP_NoCopy.h"
#include "CP_OSStatus.h"
#include "CP_OSTypes.h"
#include "CP_SignalSlot.h"
#include "CP_SignalSlot_Defines.h"
#include "CP_String.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

// ===================================================================================
//	CP_S_MenuSelection - structure used to pass a menu selection in the CP_MenuSelectionSignal.
// ===================================================================================
struct CP_S_MenuSelection
{
	OSMenu					fOSMenu;
	UInt16					fMenuItem;
	
							CP_S_MenuSelection();
							CP_S_MenuSelection(
									OSMenu inOSMenu,
									UInt16 inMenuItem );
							CP_S_MenuSelection( const CP_S_MenuSelection& inMenuSelection );

	CP_S_MenuSelection&		operator =( const CP_S_MenuSelection& inMenuSelection );
};

/*!
*	@class		CP_CommandStatus
*	@brief		Class used to obtain and set status information about a single command.
*
*	@ingroup	CommandHandling
*/
class CP_CommandStatus
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										/*!
										*	Constructor.
										*	
										*	@exception	none
										*/
										CP_CommandStatus()
											: fIsEnabled(false)							{};
										CP_CommandStatus( const CP_CommandStatus& inOutStatus );
	//@}
									
	//! @name Enabled
	//@{
										/*!
										*	Returns true if the command should be enabled.
										*	
										*	@exception	none
										*
										*	@return		bool			Returns true if the command is enabled.
										*/
			bool						IsEnabled()										{ return( fIsEnabled ); }

										/*!
										*	Sets the enabled status for this command.
										*	
										*	@param		inIsEnabled		True if this command should be enabled.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetIsEnabled( bool inIsEnabled )				{ fIsEnabled = inIsEnabled; }
	//@}
			
	//! @name operators
	//@{
			CP_CommandStatus&			operator =( const CP_CommandStatus& inOutStatus );
	//@}

	//! @name Text
	//@{
										/*!
										*	Returns true if there is any text associated with this command.
										*
										*	@exception	none
										*
										*	@return		bool			True if this command has any text associated with it.
										*/
			bool						HasText()										{ return( fItemText.length() > 0 ); }

										/*!
										*	Sets the text associated with this command. This only needs to be called if the command
										*	alters a menu, such as undo / redo, otherwise the text is not used.
										*
										*	@param		inText			The text associated with this command.
										*
										*	@exception	none
										*
										*	@return		void
										*/
			void						SetText( const CP_String& inText )				{ fItemText = inText; }

										/*!
										*	Returns the text associated with this command. This is used for commands, such as
										*	undo / redo that alter the text of a menu.
										*
										*	@exception	none
										*
										*	@return		CP_String const		The text associated with this command.
										*/
			CP_String					GetText() const									{ return( fItemText ); }
	//@}

	// class data
private:
	bool								fIsEnabled;		/*! Flag to indicate if the command is enabled. */
	CP_String							fItemText;		/*! Text associated with this command. */
};

/*!	\var typedef CP_CommandStatusSignal
 *	Signal typedef used to handle command status signal/slot connections. 
 */
typedef CP_SigSlot::CP_Signal3<CP_CommandStatus&, bool&, CP_CommandNo&>			CP_CommandStatusSignal;

/*!	\var typedef CP_CommandHandlerSignal
 *	Signal typedef used to handle command handler signal/slot connections. 
 */
typedef CP_SigSlot::CP_Signal2<OSStatus&, CP_CommandNo&>						CP_CommandHandlerSignal;

/*!	\var typedef CP_MenuSelectionSignal
 *	Signal typedef used to handle sending a signal for a menu selection with no associated command. 
 */
typedef CP_SigSlot::CP_Signal1<const CP_S_MenuSelection&>						CP_MenuSelectionSignal;

/*!
*	@class		CP_Command
*	@brief		Class used to handle a single command.
*
*	@ingroup	CommandHandling
*/
class CP_Command
{
public:
	/*! Flags used to set and get the status of commands. */
	enum
	{
		CP_E_CH_UseCommandStatus			= 1,		/*!< Command uses the CP_CommandStatus class to obtain information. */
		CP_E_CH_AlwaysEnable				= 2,		/*!< Always enable the commnand. */
		CP_E_CH_AlwaysDisable				= 4,		/*!< Always disable the command. */
		
			// application state flags
		CP_E_CH_DisableIfLowMemory			= 8,		/*!< Disable the command because of low memory. */
		CP_E_CH_DisableIfModalDialog		= 0x10,		/*!< Disable the command because of a modal dialog being displayed. */
		CP_E_CH_EnableIfDocWindowTopmost	= 0x20		/*!< Enable the command if the top most window is a CPLAT document window. */
	};

	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Command();
										CP_Command( const CP_Command& inCommand );
	//! Destructor.
										~CP_Command();
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the CP_CommandNo associated with this command.
										*
										*	@exception	none
										*
										*	@return		CP_CommandNo	The CP_CommandNo associated with this command.
										*/
		CP_CommandNo					GetCommand()									{ return( fCommand ); }

										/*!
										*	Returns any flags associated with this command.
										*
										*	@exception	none
										*
										*	@return		SInt32			Flags associated with this command.
										*/
		SInt32							GetCommandFlags()								{ return( fCommandFlags ); }

										/*!
										*	Returns the command handler signal associated with this command. This is the
										*	signal that will be called when the user selects this command from a menu, or
										*	otherwise causes the command number to be sent. The command and signal need to
										*	be registered with the CP_CommandHanlingManager first.
										*
										*	@exception	none
										*
										*	@return		CP_CommandHandlerSignal		Signal emited with this command.
										*/
		CP_CommandHandlerSignal			*GetCommandHandlerSignal()						{ return( fCommandHandler ); }

										/*!
										*	Returns the command status signal associated with this command. This is the
										*	signal that will be called whenn the status for this command needs to be updated,
										*	such as a menu opening. The command and signal need to be registered with the
										*	CP_CommandHanlingManager first. You can use this inplace of the CP_E_CH enumeration
										*	above to determine the status of enabled commands, if you need finer control than
										*	having a command always enabled or disabled, such as when the state of the command
										*	is determined by runtime conditions.
										*
										*	@exception	none
										*
										*	@return		CP_CommandHandlerSignal		Signal emited with this command.
										*/
		CP_CommandStatusSignal			*GetCommandStatusHandlerSignal()				{ return( fCommandStatus ); }
	//@}

	//! @name Operators
	//@{
		CP_Command&						operator =( const CP_Command& inCommand );
	//@}
									
	//! @name Setters
	//@{
										/*!
										*	Sets the CP_CommandNo associated with this command.
										*
										*	@param		inCommand		The CP_CommandNo associated with this command.
										*
										*	@exception	none
										*
										*	@return		void
										*/
		void							SetCommand( CP_CommandNo inCommand )								{ fCommand = inCommand; }

										/*!
										*	Flags associated with this command and used to determine the status of the command. The
										*	flags should be one of the following:
										*		- CP_E_CH_UseCommandStatus
										*		- CP_E_CH_AlwaysEnable
										*		- CP_E_CH_AlwaysDisable
										*		- CP_E_CH_DisableIfLowMemory
										*		- CP_E_CH_DisableIfModalDialog
										*		- CP_E_CH_EnableIfDocWindowTopmost
										*
										*	@param		inFlags		Flags associated with this command.
										*
										*	@exception	none
										*
										*	@return		void
										*/
		void							SetCommandFlags( SInt32 inFlags )									{ fCommandFlags = inFlags; }

										/*!
										*	Sets the signal associated with this command when it is selected, or sent by a control.
										*
										*	@param		inSignal		Signal sent when this command is selected or sent.
										*
										*	@exception	none
										*
										*	@return		void
										*/
		void							SetCommandHandlerSignal( CP_CommandHandlerSignal *inSignal )		{ fCommandHandler = inSignal; }

										/*!
										*	Sets the signal associated with this command needs to be enabled. This is used when you need
										*	finer control than that provided by the SetCommandFlags, such as when the state of a command
										*	is dependent upon some runtime conditionn.
										*
										*	@param		inSignal		Signal sent when this command status needs updating.
										*
										*	@exception	none
										*
										*	@return		void
										*/
		void							SetCommandStatusSignal( CP_CommandStatusSignal *inStatusSignal )	{ fCommandStatus = inStatusSignal; }
	//@}

	// class data
private:
	CP_CommandNo						fCommand;				/*! CP_CommandNo assigned for this command. */
	SInt32								fCommandFlags;			/*! Flags used to find information out about the status of the command. */
	CP_CommandHandlerSignal				*fCommandHandler;		/*! Signal called to handle the command. */
	CP_CommandStatusSignal 				*fCommandStatus;		/*! Signal called to update the status of the command. */
};

/*!	\var typedef CP_Commands_Array
 *	vector that contains a list of CP_Commands to be handled. 
 */
/*!	\var typedef CP_Commands_Iterator
 *	forward iterator for the CP_Commands_Array. 
 */
/*!	\var typedef CP_Commands_ReverseIterator
 *	reverse iterator for the CP_Commands_Array. 
 */
#if TARGET_OS_LINUX
	typedef std::vector<CP_Command *>															CP_Commands_Array;
	typedef std::vector<CP_Command *>::iterator													CP_Commands_Iterator;
	typedef std::vector<CP_Command *>::reverse_iterator											CP_Commands_ReverseIterator;
#else
	typedef std::vector<CP_Command *, std::allocator<CP_Command *> >							CP_Commands_Array;
	typedef std::vector<CP_Command *, std::allocator<CP_Command *> >::iterator					CP_Commands_Iterator;
	typedef std::vector<CP_Command *, std::allocator<CP_Command *> >::reverse_iterator			CP_Commands_ReverseIterator;
#endif

/*!
*	@class		CP_M_CommandHandler
*	@brief		Singleton class that manages the registration and lookup of commands.
*
*				This class is used to register CP_CommandNos and their associated flags, or signals
*				that are used when the user causes some action that will cause the command to
*				be called.
*
*	@ingroup	CommandHandling
*/
class CP_EXPORT CP_M_CommandHandler : public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_M_CommandHandler();
	//! Destructor.
	virtual								~CP_M_CommandHandler();
	//@}
			
	//! @name Command Status
	//@{
			bool						UpdateCommandStatus(
												CP_CommandNo inOSCommand,
												CP_CommandStatus& inOutCommandStatus );
	virtual	bool						UpdateCommandStatusForAppState(
												CP_CommandNo inOSCommand,
												CP_CommandStatus& inOutCommandStatus,
												CP_Command *inCommand );
	//@}

	//! @name Command Handling
	//@{
			OSStatus					HandleCommand( CP_CommandNo inOSCommand );
	//@}
										
	//! @name Getters
	//@{
										/*!
										*	Returns the signal called when a menu item is selected and has no
										*	command associated with it.
										*	
										*	@exception	none
										*	
										*	@return		CP_MenuSelectionSignal *	Returns the menu selection signal.
										*/
			CP_MenuSelectionSignal		*GetMenuSelectionSignal()									{ return( fMennuSelectionSignal ); }
	//@}

	//! @name Registration
	//@{
			void						RegisterCommand(
												CP_CommandNo inOSCommand,
												CP_CommandHandlerSignal *inCommandHandler,
												CP_CommandStatusSignal *inCommandStatus );
			void						RegisterCommand(
												CP_CommandNo inOSCommand,
												CP_CommandHandlerSignal *inCommandHandler,
												SInt32 inCommandFlags );
			CP_CommandHandlerSignal 	*RegisterCommand(
												CP_CommandNo inOSCommand,
												SInt32 inCommandFlags );
			CP_CommandStatusSignal 		*RegisterUpdateStatusHandler( CP_CommandNo inOSCommand );
			CP_CommandHandlerSignal		*RegisterCommandHandler( CP_CommandNo inOSCommand );
			bool						IsRegisteredCommand( CP_CommandNo inCommand ) const;
			void						ReplaceCommand(
												CP_CommandNo inOSCommand,
												CP_Command *inCommand );
			void						UnRegisterCommand( CP_CommandNo inOSCommand );
					
			CP_Command					*GetCommand( CP_CommandNo inOSCommand );
	//@}

	// class data
protected:
	CP_Commands_Array					*fCommands;						/*! List of commands managed by this class. */
	
	CP_MenuSelectionSignal				*fMennuSelectionSignal;			/*! Signal called when a menu item is selected with no associated command. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
#if !CP_CORE_ONLY // JUM 2004-11-25
	//! @name Command Handling
	//@{
			OSStatus					HandleCommand(
												HICommand inCommand,
												CP_Menu *inMenu = NULL );
	//@}

	//! @name Command Status
	//@{
			OSStatus					HandleCommandUpdateStatus(
												HICommand inCommand,
												CP_Menu *inMenu = NULL );
			OSStatus					UpdateCommandStatusOS(
												CP_CommandStatus inCommandStatus,
												bool inResult,
												HICommand inCommand );
	//@}
#endif // CP_CORE_ONLY
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

#define	CP_RegisterCommand(inOSCommand, inCommandFlags)					RegisterCommand( inOSCommand, inCommandFlags )

#define	CP_RegisterUpdateStatusHandler(inOSCommand)						RegisterUpdateStatusHandler( inOSCommand )
#define	CP_RegisterCommandHandler(inOSCommand)							RegisterCommandHandler( inOSCommand )

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_CommandHandler

