//=========================================================================
//
//	FILE:				CP_FileSystem.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Routines for dealing with the platforms File System.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 204 $
//						Latest Checkout:  $Date: 2005-06-06 22:15:38 -0700 (Mon, 06 Jun 2005) $
//
//=========================================================================
#ifndef _H_CP_FileSystem
#define _H_CP_FileSystem
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Date.h"
#include "CP_FileSystemSpec.h"
#include "CP_FileSystemSpec.h"
#if !CP_CORE_ONLY // JUM 2005-01-07
#include "CP_OSIcon.h"
#endif
#include "CP_OSStatus.h"
#include "CP_OSTypes.h"
#include "CP_String.h"
#include "CP_Time.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_OS_WIN32
	#if UNICODE
		const OSVolume kOnSystemDisk = L"c";
	#else
		const OSVolume kOnSystemDisk = "c";
	#endif
#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX
	const OSVolume kOnSystemDisk = NULL;
#endif	// TARGET_OS_LINUX

/*!
*	@class		CP_FileSystem - [namespace]
*	@brief		A collection of of routines to handle various file system related tasks.
*	
*				The routines in this namespace handle misc tasks for the platforms
*				file system that are not handled by the CP_File class. 
*
*	@ingroup	FileSystem
*/

namespace CP_FileSystem
{
	//! @name Files and Folders
	//@{
			void						DeleteObject( const CP_FileSystemSpec& inOutFileSpec, bool inNotifyFinder=true);
			bool						SetLocked(const CP_FileSystemSpec& inFileSpec,bool inLockit);
			bool						SetReadOnly( const CP_FileSystemSpec& inFileSpec, bool inReadOnly );
	//@}
	
	//! @name Files
	//@{
			void						DeleteFile( const CP_FileSystemSpec& inOutFileSpec, bool inNotifyFinder=true);
			void						RenameFile( const CP_FileSystemSpec& inFileSpec, const CP_String& inNewName, CP_FileSystemSpec* outRenamedFileSpec, bool inNotifyFinder=true);
			void						RenameFile( CP_FileSystemSpec& inOutFileSpec, const CP_String& inNewName, bool inNotifyFinder=true);
			CP_String					GetExtension( const CP_FileSystemSpec& inFileSpec );
			CP_FileSystemSpec			SetExtension(
												const CP_FileSystemSpec& inFileSpec,
												const CP_String& inFileExtension );
			bool						HasExtension(
												const CP_FileSystemSpec& inFileSpec,
												const CP_String& inFileExtension );

											// File Info
			SInt64						GetSize( const CP_FileSystemSpec& inFileSpec );

			FourCharCode				GetType( const CP_FileSystemSpec& inFileSpec );
			void						GetTypeAndCreator( const CP_FileSystemSpec& inFileSpec,FourCharCode *outType,FourCharCode* outCreator);

			CP_Date						GetCreatedDate( const CP_FileSystemSpec& inFileSpec );
			CP_Time 					GetCreatedTime( const CP_FileSystemSpec& inFileSpec );

			CP_Date						GetModifiedDate( const CP_FileSystemSpec& inFileSpec );
			CP_Time 					GetModifiedTime( const CP_FileSystemSpec& inFileSpec );

#if !CP_CORE_ONLY // JUM 2005-01-07
											// File Icons
			CP_OSIcon					GetLargeIcon( const CP_FileSystemSpec& inFileSpec );
			CP_OSIcon					GetSmallIcon( const CP_FileSystemSpec& inFileSpec );
#endif
			
			CP_FileSystemSpec			CreateTempFile(const CP_String& inFileNameTemplate=CP_String::emptyString, const CP_FileSystemSpec* inParentFolder=NULL);
	//@}
									
	//! @name Folders
	//@{
			CP_FileSystemSpec			CreateFolder( const CP_FileSystemSpec& inParentFolderSpec, const CP_String inFolderName, bool inNotifyFinder=true);
			CP_FileSystemSpec			CreateFolder(const CP_FileSystemSpec& inFolderSpec, bool inNotifyFinder=true);
			void						RemoveFolder( const CP_FileSystemSpec& inOutFileSpec, bool inNotifyFinder=true);
			bool						FolderExists( const CP_FileSystemSpec& inFolderSpec );

			CP_FileSystemSpec			GetFolder(
												FourCharCode inFolderType,
												const OSVolume& inVolume = kOnSystemDisk );
			CP_FileSystemSpec			GetTempFolder( const OSVolume& inVolume = kOnSystemDisk );
			CP_FileSystemSpec			GetTempFileSpec(
												const CP_String& inFileName,
												const CP_String& inFileExtension );
			CP_FileSystemSpec			CreateTempFolder(const CP_String& inFolderNameTemplate=CP_String::emptyString);
	//@}
		
	//! @name Volumes
	//@{
			bool						VolumeExists( const CP_String& inVolumeName );
	//@}

	//! @name Testers
	//@{
			bool						IsWriteable( const CP_FileSystemSpec& inFileSpec );
			bool						ExistsOnDisk( const CP_FileSystemSpec& inFileSpec );
	//@}
	
	//========================================================================
	// Platform specific routines
	//========================================================================
	#if TARGET_OS_MAC
	//! @name Testers
	//@{
		bool							HasResourceFork( const CP_FileSystemSpec& inFileSpec );
	//@}

	//! @name Creator and Type
	//@{
			FourCharCode				GetCreator( const CP_FileSystemSpec& inFileSpec );

			void						SetCreatorAndType(
												const CP_FileSystemSpec& inFileSpec,
												FourCharCode inCreator,
												FourCharCode inType );
			void						SetCreator(
												const CP_FileSystemSpec& inFileSpec,
												FourCharCode inCreator );
										
			void						SetType(
												const CP_FileSystemSpec& inFileSpec,
												FourCharCode inType );
	//@}
	
			//! Make an FSRef and be smart about parentRef pointing to symlink.
	OSStatus							FSMakeFSRefUnicodeLinkSafe(
												const FSRef *    parentRef,
												UniCharCount     nameLength,
												const UniChar *  name,
												TextEncoding     textEncodingHint,
												FSRef *          newRef) throw();
	
	OSStatus							FollowSymlink(const FSRef& inFileRef,
													  FSRef* outResolvedFileRef,
													  bool *outWasSymlink=NULL,
													  bool *outIsFolder=NULL) throw();
	
#if !CP_CORE_ONLY // JUM 2005-01-07
	//! @name File Info
	//@{
			CP_OSIcon					GetOSLargeIcon( const CP_FileSystemSpec& inFileSpec );
			CP_OSIcon					GetOSSmallIcon( const CP_FileSystemSpec& inFileSpec );
			CP_OSIcon					GetIconServicesIcon( const CP_FileSystemSpec& inFileSpec );
	//@}
#endif // CP_CORE_ONLY
	#endif

	#if TARGET_OS_WIN32
 #if !CP_CORE_ONLY // JUM 2005-01-07
	//! @name File Info
	//@{
			CP_OSIcon					GetOSLargeIcon( const CP_FileSystemSpec& inFileSpec );
			CP_OSIcon					GetOSSmallIcon( const CP_FileSystemSpec& inFileSpec );
	//@}
	
 #endif // CP_CORE_ONLY
	//! @name File operation
	//@{
			bool						RecursiveDeleteDirectory( const CP_String& inDirectory );
	//@}
	#endif

	#if TARGET_OS_LINUX
#if !CP_CORE_ONLY // JUM 2005-01-07
	//! @name File Info
	//@{
			CP_OSIcon					GetOSLargeIcon( const CP_FileSystemSpec& inFileSpec );
			CP_OSIcon					GetOSSmallIcon( const CP_FileSystemSpec& inFileSpec );
	//@}
#endif // CP_CORE_ONLY
	#endif
	
}	// namespace CP_FileSystem

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FileSystem

