//=========================================================================
//
//	FILE:				CP_View.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Base class for all UI elements in the CPLAT framework.
//						All UI elements derive from this common base class, and
//						implement 2 basic types of UI elements: Platform controls
//						are OS defined elements, and user defined elements.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 258 $
//						Latest Checkout:  $Date: 2005-08-27 13:57:43 -0700 (Sat, 27 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_View
#define _H_CP_View
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_M_SubViewHandler.h"
#include "CP_Point.h"
#include "CP_Printer_PageFormat.h"
#include "CP_Size.h"
#include "CP_View_Imp.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_View
*	@brief		Base class for all UI elements in the CPLAT framework.
*	
*				All UI elements derive from this common base class, and implement 2 basic
*				types of UI elements:
*
*				Platform controls are OS defined elements, and user defined elements. The
*				actual implementation of the CP_View class is done by an implemntation
*				class that is platform specific.
*
*	@ingroup	UI
*/

	// forward class declarations
class CP_Printer;

class CP_EXPORT CP_View : public CP_View_Imp, public CP_M_SubViewHandler
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												bool inCreateImp = false,
												UInt32 inFlags = CP_E_ViewFlags_None );
	//@}
	//! Destructor.
	virtual								~CP_View() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Appearance
	//@{
			void						Show();
			void						Hide();

			void						Activate();
			void						Deactivate();

			void						Enable();
			void						Disable();

			bool						IsFocused()									{ return( fIsFocused ); }
			void						SetFocused( bool inFocused );
	virtual	bool						CanLoseFocus();
			bool						FocusCanChange();
			void						LosingFocus();
	//@}

	//! @name Events
	//@{
			void						HandleActivationEvent();
			void						HandleDeactivationEvent();
	//@}


	//! @name Frame
	//@{
	virtual	void						HandleBoundsChanged(
												const CP_Rect& inPreviousBounds,
												const CP_Rect& inNewBounds );
	//@}

	//! @name Getters
	//@{
			OSControl					GetOSControl();
	//@}

	//! @name Help
	//@{
	virtual	void						SetHelp(
												const CP_String& inHelpString,
												const CP_String& inSecondaryHelpString );
	virtual	void						SetHelp(
												SInt16 inHelpStringResID,
												SInt16 inHelpStringIndex,
												SInt16 inSecondaryHelpStringResID = 0,
												SInt16 inSecondaryHelpStringIndex = 0 );
	//@}

	//! @name Movement
	//@{
	virtual	void						Move( const CP_Point& inDistance );
	//@}

	//! @name Positioners
	//@{
			void						SetLocation( const CP_Point& inLocation );
			void						SetSize( const CP_Size& inSize );
	//@}

	//! @name Printing
	//@{
	virtual	void						OnBeginPrinting( CP_Printer */*inPrinter*/ )	{}
	virtual	void						OnPrint(
												CP_Canvas& inCanvas,
												UInt32 inPageNumber,
												CP_Printer_PageFormat pageFormat );
	virtual	void						OnEndPrinting( CP_Printer */*inPrinter*/ )		{}

	virtual	void						CountTiles(
												CP_Printer *inPrinter,
												CP_Printer_PageFormat& pageFormat,
												UInt16 *inOutHorTiles,
												UInt16 *inOutVerTiles );

	CP_Point							GetPrintLocation()								{ return( fPrintLocation ); }
	CP_Size								GetPrintSize()									{ return( fPrintSize ); }
	CP_Rect								GetLocalPrintBounds();
	
			void						SetPrintLocation( const CP_Point& inLocation )	{ fPrintLocation = inLocation; }
			void						SetPrintSize( const CP_Size& inSize )			{ fPrintSize = inSize; }
	//@}

	//! @name Superview
	//@{
	virtual	void						AddSubview( CP_View *inView );
	//@}

	//! @name Testers
	//@{
			bool						IsVisible() const;
			bool						IsActive() const;
			bool						IsEnabled() const;
			
			bool						IsContainer() const								{ return( fIsContainer ); }
	//@}

protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_View();
	//@}

	//! @name Focus
	//@{
	virtual	void						OnLosingFocus();
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	virtual	void						OnFinishPersist();
	//@}

	// class data
protected:
	bool								fIsContainer;
	
	#if TARGET_OS_WIN32
		OSWindow						fToolTipWindow;
	#endif
	
	#if TARGET_OS_LINUX
		GtkWidget						*fContentAreaContainer;		/*! Gtk container widget that all subviews are contained in
																		for this view. */
	#endif
	
	CP_Point							fPrintLocation;
	CP_Size								fPrintSize;
	
	bool								fIsFocused;
	CP_DataValueSignal					fFocusChangedSignal;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
			void 						InitFromOSControl(
												CP_View *inSuperview,
												OSControl inOSControl );

	virtual	void						SetForeBackColors();
	
protected:
	#if TARGET_API_MAC_CARBON && TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020
			void						AddHIViewSubview(
												CP_View *inSuperview,
												HIViewRef inHIViewRef );
	#endif

	virtual	void						FinishInitFromOSControl( OSControl /*inOSControl*/ )	{}
#endif

#if TARGET_OS_WIN32
public:
	virtual	LRESULT						HandleWinHelperProcMessage(
												HWND hwnd,
												UINT iMsg,
												WPARAM wParam,
												LPARAM lParam );

			void						DrawViewHierachy();
			void						SetLocationAndSubs( const CP_Point& inLocation );
#endif

#if TARGET_OS_LINUX
public:
	//! @name Container
	//@{
    		GtkWidget					*GetContentAreaContainer()										{ return( fContentAreaContainer ); }
    		void						SetContentAreaContainer( GtkWidget *inContentAreaContainer )	{ fContentAreaContainer = inContentAreaContainer; }
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Viewo

